
/* 
 * This file is part of the RTL examples distribution (https://tealsemi.com).
 * Copyright (c) 2022 Franz Steininger.
 * 
 * This program is free software: you can redistribute it and/or modify  
 * it under the terms of the GNU General Public License as published by  
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

`default_nettype none

module test_ppa (
  input  wire signed [15:0] a,   // input term1 (real part)
  input  wire signed [15:0] b,   // input term1 (imaginary part)
  input  wire signed [15:0] c,   // input term2 (real part)
  input  wire signed [15:0] d,   // input term2 (imaginary part)
  output wire signed [32:0] x,   // result of product (real part)
  output wire signed [32:0] y    // result of product (imaginary part)
);



  // alternative code (Gauss method) using 3 multiplications with extra additions
  //     t1 = a * c
  //     t2 = b * d
  //     t3 = (a+b) * (c+d)
  //     x  = t1 - t2 (real)
  //     y  = t3 - t1 - t2 (imaginary)


wire signed [31:0] t1;
wire signed [31:0] t2;
wire signed [33:0] t3;
wire signed [16:0] s1;
wire signed [16:0] s2;
wire signed [34:0] y_xxl;

assign t1 = a * c;
assign t2 = b * d;
assign s1 = a + b;
assign s2 = c + d;

/* optional pipeline stage */

assign x = t1 - t2;
assign y_xxl = s1 * s2 - t1 -t2;
assign y = $signed(y_xxl[32:0]);

endmodule

`default_nettype wire


